//+------------------------------------------------------------------+
//|                                                         LPCD.mq4 |
//|                                                       Antisyzygy |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Antisyzygy"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Yellow 
#property indicator_width1 3

//---- Indicator Parameters
extern int     Price               = 0;  //Price mode : 0-Close,1-Open,2-High,3-Low,4-Median,5-Typical,6-Weighted
extern int     Order               = 3;  //Filter Order: 1-EMA,2-2nd Order,3-3rd Order
extern int     FilterPeriodFast    = 12;  //Filter Period of Fast
extern double  PctFilterFast       = 0;  //Dynamic filter in decimal(multiplier for StdDev)
extern int     FilterPeriodSlow    = 24; //Filter Period of Slow
extern double  PctFilterSlow       = 0;  //Dynamic filter in decimal(multiplier for StdDev)
extern int     PreSmooth           = 1;  //Pre-smoothing period
extern int     PreSmoothMode       = 0;  //Pre-smoothing MA Mode: 0-SMA,1-EMA,2-SMMA,3-LWMA

//---- Indicator Buffers
double     LPCD[];
double     FastFilter[];
double     SlowFilter[];
double     Smoother[];
double     DelSlow[];
double     AvgDelSlow[];
double     DelFast[];
double     AvgDelFast[];

int        draw_begin;

//+------------------------------------------------------------------+
//| LowPassFilter Convergence Divergence initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- Indicator buffers mapping
   IndicatorBuffers(8);
   SetIndexBuffer(0,LPCD);
   SetIndexBuffer(1,SlowFilter);
   SetIndexBuffer(2,FastFilter);
   SetIndexBuffer(3,Smoother); 
   SetIndexBuffer(4,DelSlow);
   SetIndexBuffer(5,AvgDelSlow);
   SetIndexBuffer(6,DelFast);
   SetIndexBuffer(7,AvgDelFast);
//---- Drawing settings

   SetIndexStyle(0,DRAW_HISTOGRAM);
   draw_begin = FilterPeriodSlow + PreSmooth;              //Calculate the beginning of the drawing area
   SetIndexDrawBegin(0,draw_begin);                      
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);

//---- Name for DataWindow and Indicator subwindow label
   IndicatorShortName("LPFilt Conv-Div : Order : "+Order+", (Slow,Fast) : ("+FilterPeriodSlow+","+FilterPeriodFast+") : ");
   SetIndexLabel(0,"LowPassFilter");

//---- Initialization done
   return(0);
}


//+------------------------------------------------------------------+
//| LowPassFilter Convergence Divergence                             |
//+------------------------------------------------------------------+

int start()
{
   int limit, i, shift;
   int counted_bars=IndicatorCounted();
   double a, b, c, pi=3.1415926535;
   
   if(counted_bars<1)
   for(i=1;i<=draw_begin;i++) 
   {
      SlowFilter[Bars-i]=0; 
      FastFilter[Bars-i]=0; 
   }

//---- Last counted bar will be recounted
   if(counted_bars>0) counted_bars--;

//---- Limit for the 
   limit=Bars-counted_bars;

   for(shift=limit; shift>=0; shift--)
   Smoother[shift] = iMA(NULL,0,PreSmooth,0,PreSmoothMode,Price,shift);
 
   for(shift=limit; shift>=0; shift--)
   {
      if(Order == 1)
      {
         SlowFilter[shift] = iMAOnArray(Smoother,0,FilterPeriodSlow,0,1,shift);
         FastFilter[shift] = iMAOnArray(Smoother,0,FilterPeriodFast,0,1,shift);
      } 
      else
		if(Order == 2) 
		{
		    //Slow filter calculations
		    a = MathExp(-MathSqrt(2)*pi/FilterPeriodSlow);
		    b = 2*a*MathCos(MathSqrt(2)*pi/FilterPeriodSlow);
    	    SlowFilter[shift] = b*SlowFilter[shift+1] - a*a*SlowFilter[shift+2] + (1 - b + a*a)*Smoother[shift];
    	    
    	    //Fast filter calculations
    	    a = MathExp(-MathSqrt(2)*pi/FilterPeriodFast);
		    b = 2*a*MathCos(MathSqrt(2)*pi/FilterPeriodFast);
    	    FastFilter[shift] = b*FastFilter[shift+1] - a*a*FastFilter[shift+2] + (1 - b + a*a)*Smoother[shift];
		}
		else
		if(Order == 3) 
		{
		    //Slow filter calculations
		    a = MathExp(-pi/FilterPeriodSlow);
		    b = 2*a*MathCos(MathSqrt(3)*pi/FilterPeriodSlow);
    	    c = MathExp(-2*pi/FilterPeriodSlow); //a * a;
    	    SlowFilter[shift] = (b+c)*SlowFilter[shift+1] - (c+b*c)*SlowFilter[shift+2] + c*c*SlowFilter[shift+3] + (1-b+c)*(1-c)*Smoother[shift];
    	    
    	    //Fast filter calculations
		    a = MathExp(-pi/FilterPeriodFast);
		    b = 2*a*MathCos(MathSqrt(3)*pi/FilterPeriodFast);
    	    c = MathExp(-2*pi/FilterPeriodFast); //a * a;
    	    FastFilter[shift] = (b+c)*FastFilter[shift+1] - (c+b*c)*FastFilter[shift+2] + c*c*FastFilter[shift+3] + (1-b+c)*(1-c)*Smoother[shift];
		}
  
   int Length1 = FilterPeriodSlow;
   int Length2 = FilterPeriodFast;
      
      //Slow PCTFilter
      if (PctFilterSlow>0)
      {
         DelSlow[shift] = MathAbs(SlowFilter[shift] - SlowFilter[shift+1]);
   
         double sumdelS=0;
         for (int j=0;j<=Length1-1;j++) sumdelS = sumdelS+DelSlow[shift+j];
         AvgDelSlow[shift] = sumdelS/Length1;
    
         double sumpowS = 0;
         for (j=0;j<=Length1-1;j++) sumpowS+=MathPow(DelSlow[j+shift]-AvgDelSlow[j+shift],2);
         double StdDevS = MathSqrt(sumpowS/Length1); 
     
         double filterS = PctFilterSlow * StdDevS;
     
         if(MathAbs(SlowFilter[shift]-SlowFilter[shift+1]) < filterS ) SlowFilter[shift]=SlowFilter[shift+1];
      }
      else
      filterS=0;
      
      //Fast PCTFilter
      if (PctFilterFast>0)
      {
         DelFast[shift] = MathAbs(FastFilter[shift] - FastFilter[shift+1]);
   
         double sumdelF=0;
         for (int k=0;k<=Length2-1;k++) sumdelF = sumdelF+DelFast[shift+k];
         AvgDelFast[shift] = sumdelF/Length2;
    
         double sumpowF = 0;
         for (k=0;k<=Length2-1;k++) sumpowF+=MathPow(DelFast[k+shift]-AvgDelFast[k+shift],2);
         double StdDevF = MathSqrt(sumpowF/Length2); 
     
         double filterF = PctFilterFast * StdDevF;
     
         if(MathAbs(FastFilter[shift]-FastFilter[shift+1]) < filterF ) FastFilter[shift]=FastFilter[shift+1];
      }
      else
      filterF=0;
         
      // Assign the convergence / divergence
		LPCD[shift]=FastFilter[shift]-SlowFilter[shift];
		
		// Change color for downward/upward trend
		if(LPCD[shift]<0)
		{
		}
		else
		{
		}
   }
//---- done
   return(0);

}